from cocos.sprite import *
import pyglet
from pyglet.image import Animation, AnimationFrame
import random

class Enemy(Sprite):
    def __init__(self,posX, posY, turn, isAir):
        self.rand = random.randint(0,2)
        self.left = turn
        if isAir == False:
            if self.rand == 0:
                animation = AnimationLibrary.create_effect_animation('Enemy','enemy01',8)
            elif self.rand == 1:
                animation = AnimationLibrary.create_effect_animation('Enemy','enemy02',4)
            else:
                animation = AnimationLibrary.create_effect_animation('Enemy','enemy03',7)
        else:
            animation = AnimationLibrary.create_effect_animation('Enemy','airEnemy',2)
        if turn == 0:
            animation = animation.get_transform(True,False,0)
        super(Enemy,self).__init__(animation)
        self.position = (posX, posY)
        if (isAir == False):
            self.size = 1.25-(posY/800)
            
class Explosion(Sprite):
    def __init__(self,posX, posY):
        animation = AnimationList.explosion01
        super(Explosion,self).__init__(animation)
        self.time = 0
        self.position = (posX,posY)
        self.schedule(self.loop)
    def loop(self,dt):
        self.time += 1
        if self.time >= 30:
            self.kill()
class Corpse(Sprite):
    def __init__(self,posX,posY,isLeft,size):
        self.rand = random.randint(1,2)
        self.left = 0
        self.time = 0
        
        
        if isLeft == 0:
            self.left = -1
        else:
            self.left = 1
        if self.rand == 1:
            animation = AnimationList.death01
            #animation = AnimationLibrary.create_unloop_effect('Corpse','death01',16)
        else:
            animation = AnimationList.death02
            #animation = AnimationLibrary.create_unloop_effect('Corpse','death02',17)
        if self.left == -1:
            animation = animation.get_transform(True,False,0)
        super(Corpse,self).__init__(animation)
        if self.rand == 1:
            self.position = (posX + (self.left*size*100), posY - (size*100))
        else:
            self.position = (posX + (self.left*size*200), posY)
        self.scale = size
        self.schedule(self.loop)
    def loop(self,dt):
        self.time += 1
        if (self.time > 120):
            self.opacity -= 8
        if self.opacity <= 0:
            self.kill()
class AirEnemy(Sprite):
    def __init__(self,posX, posY, turn):
        animation = AnimationList.airEnemy
        if turn == 0:
            animation = animation.get_transform(True,False,0)
        super(AirEnemy,self).__init__(animation)
        self.position = (posX, posY)
        
class AnimationLibrary(object):
    @staticmethod
    def create_effect_animation(path,image_name, frame):  
        imgList =  []
        for i in range (1,frame):
            imgList.append(pyglet.image.load('Sprite/'+path+'/'+image_name+'/'+image_name+'_'+str(i)+'.png'))
        
        effect_frames = []
        
        for effect_frame in imgList:
            effect_frames.append(AnimationFrame(effect_frame, 0.04))

        return Animation(effect_frames)
    @staticmethod
    def create_unloop_effect(path,image_name, frame):  
        imgList =  []
        for i in range (1,frame):
            imgList.append(pyglet.image.load('Sprite/'+path+'/'+image_name+'/'+image_name+'_'+str(i)+'.png'))
        
        effect_frames = []
        
        for effect_frame in imgList:
            effect_frames.append(AnimationFrame(effect_frame, 0.04))
        
        effect_frames[len(effect_frames)-1].duration = None
        return Animation(effect_frames)
    
    @staticmethod
    def create_from_sheet(path, image_name, columns, rows):  
        effect_seq = pyglet.image.ImageGrid(pyglet.image.load('Sprite/'+path+'/'+image_name+'/'+image_name+'.png'), rows, columns)
        effect_frames = []
        for row in range(rows, 0, -1):
            end = row * columns
            start = end - (columns -1) -1
            for effect_frame in effect_seq[start:end:1]:
                effect_frames.append(AnimationFrame(effect_frame, 0.04))

        effect_frames[(rows * columns) -1].duration = None
        return Animation(effect_frames)

class AnimationList():
    
    enemy01 = AnimationLibrary.create_effect_animation('Enemy','enemy01',8)
    enemy02 = AnimationLibrary.create_effect_animation('Enemy','enemy02',4)
    enemy03 = AnimationLibrary.create_effect_animation('Enemy','enemy03',7)
    airEnemy = AnimationLibrary.create_effect_animation('Enemy','airEnemy',2)
    explosion01 = AnimationLibrary.create_unloop_effect('Explosion','Explosion',12)
    death01 = AnimationLibrary.create_unloop_effect('Corpse','death01',17)
    death02 = AnimationLibrary.create_unloop_effect('Corpse','death02',18)
    
