#include <avr/io.h>
#include <avr/interrupt.h>  /* for sei() */
#include <util/delay.h>     /* for _delay_ms() */
#include <avr/pgmspace.h>   /* required by usbdrv.h */
#include "usbdrv.h"

#define RQ_SET_LED    0
#define RQ_GET_SWITCH 1
#define RQ_GET_RELOAD 3
#define RQ_GET_LIGHT 2
/* ------------------------------------------------------------------------- */
/* ----------------------------- USB interface ----------------------------- */
/* ------------------------------------------------------------------------- */
uint16_t read_adc(uint8_t channel)
{
    ADMUX = 0b01000000;
    ADMUX |= channel;
    ADCSRA = 0b11000110;
    while ((ADCSRA & (1<<ADSC))); // รอจนบิต ADSC (บิต 6) กลายเป็น 0
    return ADCL + ADCH*256;  // ผลลัพธ์ถูกเก็บอยู่ในรีจีสเตอร์ ADCL และ ADCH
}
usbMsgLen_t usbFunctionSetup(uint8_t data[8])
{
    usbRequest_t *rq = (void *)data;
    static uint8_t switch_state;  /* must stay valid when usbFunctionSetup returns */
    static uint16_t x;
    if (rq->bRequest == RQ_SET_LED)
    {
        uint8_t led_val = rq->wValue.bytes[0];
        uint8_t led_no  = rq->wIndex.bytes[0];

        if (led_val)
            PORTC |= (1<<led_no);
        else
            PORTC &= ~(1<<led_no);

        return 0;
    }

    else if (rq->bRequest == RQ_GET_SWITCH)
    {
        if ((PINC & (1<<PC3)) == 0) /* switch is pressed */
            switch_state = 1;
        else
            switch_state = 0;

        /* point usbMsgPtr to the data to be returned to host */
        usbMsgPtr = (uchar*) &switch_state;

        /* return the number of bytes of data to be returned to host */
        return sizeof(switch_state);
    }
    else if (rq->bRequest == RQ_GET_RELOAD)
    {
        if ((PINC & (1<<PC2)) == 0) /* switch is pressed */
            switch_state = 1;
        else
            switch_state = 0;

        /* point usbMsgPtr to the data to be returned to host */
        usbMsgPtr = (uchar*) &switch_state;

        /* return the number of bytes of data to be returned to host */
        return sizeof(switch_state);
    }
    else if(rq->bRequest == RQ_GET_LIGHT)
    {
        x = read_adc(PC4);
        usbMsgPtr = (uchar*) &x;
        return sizeof(x);
    }


    return 0;   /* default for not implemented requests: return no data back to host */
}

/* ------------------------------------------------------------------------- */

int main(void)
{

    DDRC  = (1<<PC0)|(1<<PC1); // PC0..PC2 = Output, PC3..PC5 = Input
    DDRC &= ~(1<<PC4);
    PORTC = (1 << PC3)|(1<< PC2);                 // Enable pull-up at PC3
    usbInit();

    /* enforce re-enumeration, do this while interrupts are disabled! */
    usbDeviceDisconnect();
    _delay_ms(300);
    usbDeviceConnect();

    /* enable global interrupts */
    sei();

    /* main event loop */
    for(;;)
    {
        usbPoll();
    }

    return 0;
}

/* ------------------------------------------------------------------------- */
